import requests
from http import HTTPStatus
from requests import Response
from gigapipe.exceptions import GigapipeServerError
from gigapipe.api_client.api import Base
from gigapipe.api_client.gigapipe_api import GigapipeApi


class Root(Base):
    """
    Root Class
    """

    def __init__(self, api):
        """
        Root Constructor
        :param api: The API instance
        """
        super(Root, self).__init__(api)

    @GigapipeApi.autorefresh_access_token
    def get_machines(self, provider_id: int, region_id: int) -> Response:
        """
        Obtains all the machines per provider and region
        :return: A list of machines
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/machines?provider={provider_id}&region={region_id}"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def get_machine(self, machine_id: int) -> Response:
        """
        Obtains a specific machine based on the id passed as a parameter
        :return: A dictionary containing the machine info
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/machines/{machine_id}"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def get_providers(self) -> Response:
        """
        Obtains all the providers
        :return: A list containing all the providers
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/providers"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def get_regions(self, provider_id: int) -> Response:
        """
        Obtains all the regions according to the provider passed as a parameter
        :return: A list containing the regions
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/regions?provider={provider_id}"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def get_disk_types(self, provider_id: int, region_id: int) -> Response:
        """
        Obtains all the disk types per provider and region
        :return: A list of disk types
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/disk-types?provider={provider_id}&region={region_id}"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response



class Base(object):
    """
    Base containing the API object. All the API instances use inherit from this class.
    """
    def __init__(self, api):
        """
        Base Constructor
        :param api: the API instance
        """
        self.api = api



from .base import Base
from .users import Users
from .organizations import Organizations
from .invites import Invites
from .roles import Roles
from .root import Root
from .clusters import Clusters
from .clickhouse import Clickhouse
from .imports import Imports
from .integrations import Integrations




from typing import Dict, Any

import requests
from http import HTTPStatus
from requests import Response
from gigapipe.exceptions import GigapipeServerError
from gigapipe.api_client.api import Base
from gigapipe.api_client.gigapipe_api import GigapipeApi


class Clusters(Base):
    """
    Clusters Class
    """

    def __init__(self, api):
        """
        Clusters Constructor
        :param api: The API instance
        """
        super(Clusters, self).__init__(api)

    @GigapipeApi.autorefresh_access_token
    def get_clusters(self) -> Response:
        """
        Obtains all the clusters for the organization the user belongs to
        :return: A list containing the clusters
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def get_cluster(self, cluster_slug: str) -> Response:
        """
        Obtains a single cluster according to the passed slug
        :param cluster_slug: the cluster slug
        :return: A dictionary containing the cluster info
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters/{cluster_slug}"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def create_cluster(self, cluster_payload: Dict[str, Any]) -> Response:
        """
        Creates a new cluster
        :param cluster_payload: the dictionary containing the necessary info to create a cluster
        :return: A message response
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters"

        try:
            response: Response = requests.post(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            }, json=cluster_payload)
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def query_cluster(self, cluster_slug: str, query: str) -> Response:
        """
        Obtains all the clusters for the organization the user belongs to
        :param cluster_slug: the cluster slug
        :param query: the clisckhose query
        :return: A dictionary containing the clickhouse response
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters/{cluster_slug}/query?query={query}"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def get_metadata(self, cluster_slug: str) -> Response:
        """
        Obtains the metadata of the cluster
        :param cluster_slug: the cluster slug
        :return: A dictionary containing the metadata
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters/{cluster_slug}/metadata"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def stop_cluster(self, cluster_slug: str) -> Response:
        """
        Stops the cluster but doesn't get rid of it
        :param cluster_slug: the cluster slug
        :return: A message response
        """
        return self._cluster_stop_resume(cluster_slug=cluster_slug, action="stop")

    @GigapipeApi.autorefresh_access_token
    def resume_cluster(self, cluster_slug: str) -> Response:
        """
        Starts a cluster that had been stopped
        :param cluster_slug: the cluster slug
        :return: A message response
        """
        return self._cluster_stop_resume(cluster_slug=cluster_slug, action="resume")

    def _cluster_stop_resume(self, cluster_slug: str, action: str):
        """
        Cluster start/stop
        :param cluster_slug: the cluster slug
        :param action: whether to start or stop
        :return: A message response
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters/{cluster_slug}/{action}"
        try:
            response: Response = requests.patch(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def get_costs(self) -> Response:
        """
        Obtains the list of costs of all the clusters of the organization
        :return: A list containing the cost per cluster
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters/costs/all"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def transfer_cluster(self, cluster_slug: str, email: str) -> Response:
        """
        Transfers the cluster to another user of the organization as long as they have permission to hold it
        :param cluster_slug: the cluster slug
        :param email: the email of the target user
        :return: A message response
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters/{cluster_slug}/transfer?email={email}"

        try:
            response: Response = requests.patch(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def scale_nodes(self, cluster_slug: str, *, payload: dict) -> Response:
        """
        Adds shards and replicas to an existing cluster
        :param cluster_slug: the cluster slug
        :param payload: the scaling payload
        :return: A message response
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters/{cluster_slug}/scale/nodes"

        try:
            response: Response = requests.post(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            }, json=payload)
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def add_disks(self, cluster_slug: str, *, payload: list[dict]) -> Response:
        """
        Adds shards and replicas to an existing cluster
        :param cluster_slug: the cluster slug
        :param payload: the scaling payload
        :return: A message response
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters/{cluster_slug}/disks"

        try:
            response: Response = requests.post(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            }, json=payload)
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def change_machine(self, cluster_slug: str, machine_id: int) -> Response:
        """
        Changes the machine of a cluster
        :param cluster_slug: the cluster slug
        :param machine_id: the id og the new machine
        :return: A message response
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters/{cluster_slug}/machine"

        try:
            response: Response = requests.patch(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            }, json={"machine_id": machine_id})
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def autoscale_disk(self, cluster_slug: str, payload: Dict[str, Any]) -> Response:
        """
        Sets the disk autoscaling for a given disk and cluster
        :param cluster_slug: the cluster slug
        :param payload: the autoscale payload
        :return: A message response
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters/{cluster_slug}/disks/autoscale"

        try:
            response: Response = requests.patch(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            }, json=payload)
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def get_autoscaling(self, cluster_slug: str, *, disk_id: int) -> Response:
        """
        Obtains the disk autoscaling for a given disk and cluster
        :param disk_id: the id of the disk
        :param cluster_slug: the cluster slug
        :return: A message response
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters/{cluster_slug}/disks/{disk_id}/autoscale"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def delete_autoscaling(self, cluster_slug: str, *, disk_id: int) -> Response:
        """
        Does away with the autoscaling for a given disk
        :param disk_id: the id of the disk
        :param cluster_slug: the cluster slug
        :return: A message response
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters/{cluster_slug}/disks/{disk_id}/autoscale"

        try:
            response: Response = requests.delete(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def delete_cluster(self, cluster_slug: str) -> Response:
        """
        Creates a new cluster
        :param cluster_slug: the cluster slug
        :return: A message response
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters/{cluster_slug}"

        try:
            response: Response = requests.delete(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response




import requests
from http import HTTPStatus
from typing import Dict, Any, Optional
from requests import Response
from gigapipe.exceptions import GigapipeServerError
from gigapipe.api_client.api import Base
from gigapipe.api_client.gigapipe_api import GigapipeApi


class Clickhouse(Base):
    """
    Clickhouse Class
    """

    def __init__(self, api):
        """
        Clickhouse Constructor
        :param api: The API instance
        """
        super(Clickhouse, self).__init__(api)

    @GigapipeApi.autorefresh_access_token
    def create_user(self, cluster_slug: str, *, user: Dict[str, str]) -> Response:
        """
        Creates a user on clickhouse
        :return: A message response
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters/{cluster_slug}/users"

        try:
            response: Response = requests.post(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            }, json=user)
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def get_users(self, cluster_slug: str) -> Response:
        """
        Obtains the clickhouse users
        :return: A message response
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters/{cluster_slug}/users"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def update_user(self, cluster_slug: str, *, user: Dict[str, Any]) -> Response:
        """
        Updates a Clickhouse user
        :return: A message response
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters/{cluster_slug}/users"

        try:
            response: Response = requests.patch(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            }, json=user)
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def delete_user(self, cluster_slug: str, username: str) -> Response:
        """
        Deletes a Clickhouse user
        :return: A message response
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters/{cluster_slug}/users?username={username}"

        try:
            response: Response = requests.delete(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def explore_tables(
        self, cluster_slug: str, *, table_name: Optional[str] = None, engine: Optional[str] = None
    ):
        """
        Displays the cluster tables according to a name and engine
        :param cluster_slug: the slug of the cluster
        :param table_name: the name of the table
        :param engine: the engine
        :return: the tables exploration
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters/{cluster_slug}/explore/tables"

        if table_name is not None and engine is None:
            url = f"{url}?table={table_name}"
        elif engine is not None and table_name is None:
            url = f"{url}?engine={engine}"
        elif table_name is not None and engine is not None:
            url = f"{url}?table={table_name}&engine={engine}"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def get_formats(self) -> Response:
        """
        Obtains the clickhouse formats
        :return: A message response
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clickhouse/formats"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response




import requests
from http import HTTPStatus
from requests import Response
from gigapipe.exceptions import GigapipeServerError

from gigapipe.api_client.api import Base
from gigapipe.api_client.gigapipe_api import GigapipeApi


class Organizations(Base):
    """
    Organizations Class
    """

    def __init__(self, api):
        """
        Organizations Constructor
        :param api: The API instance
        """
        super(Organizations, self).__init__(api)

    @GigapipeApi.autorefresh_access_token
    def get_users(self) -> Response:
        """
        Obtains the users of the organization
        :return: A dictionary containing the users of the organization
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/organizations/users"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def get_invites(self) -> Response:
        """
        Obtains the invites of the organization. (Future users who are not users yet)
        :return: A dictionary containing the invites send by the organization
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/organizations/invites"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def delete_organization(self) -> Response:
        """
        Deletes the organization
        The returned dictionary depends on whether the organization could be deleted or not:
        - organization_deleted=true and payment_link=null means the organization was deleted
        - organization_deleted=false and payment_link='some_link' means there are invoices to pay
        :return: A dictionary containing the deleted organization info
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/organizations"

        try:
            response: Response = requests.delete(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def get_upcoming_invoice(self) -> Response:
        """
        Obtains the user organization upcoming invoice
        :return: A dictionary containing the invoice values
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/organizations/upcoming-invoice"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response



import requests
from http import HTTPStatus
from typing import Dict
from requests import Response
from gigapipe.exceptions import GigapipeServerError
from gigapipe.api_client.api import Base
from gigapipe.api_client.gigapipe_api import GigapipeApi


class Integrations(Base):
    """
    Integrations Class
    """

    def __init__(self, api):
        """
        Integrations Constructor
        :param api: The API instance
        """
        super(Integrations, self).__init__(api)

    @GigapipeApi.autorefresh_access_token
    def get_integration_types(self) -> Response:
        """
        Obtains the list of integrations types
        :return: A list containing the integration types
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/integrations/types"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def get_integrations(self) -> Response:
        """
        Obtains the list of integrations
        :return: A list containing the integrations
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/integrations"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response


import requests
from http import HTTPStatus
from typing import Dict, Any
from requests import Response
from gigapipe.exceptions import GigapipeServerError
from gigapipe.api_client.api import Base
from gigapipe.api_client.gigapipe_api import GigapipeApi


class Imports(Base):
    """
    Imports Class
    """

    def __init__(self, api):
        """
        Imports Constructor
        :param api: The API instance
        """
        super(Imports, self).__init__(api)

    @GigapipeApi.autorefresh_access_token
    def import_s3_data(self, cluster_slug: str, payload: Dict[str, Any]) -> Response:
        """
        Imports S3 data to a database table
        :param cluster_slug: the cluster slug
        :param payload: the import payload
        :return: A message response
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters/{cluster_slug}/imports/s3"

        try:
            response: Response = requests.post(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            }, json=payload)
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def get_imports(self, cluster_slug: str) -> Response:
        """
        Obtains the cluster imports
        :param cluster_slug: the cluster slug
        :return: A message response
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/clusters/{cluster_slug}/imports"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response



class GigapipeError(Exception):
    """
    Gigapipe Generic Errors
    """
    def __init__(self, status_code: int, message: str):
        super(GigapipeError, self).__init__(message)
        self.status_code = status_code
        self.message = message


class GigapipeServerError(GigapipeError):
    """
    Server Errors (5xx Errors)
    """
    pass


class GigapipeClientError(GigapipeError):
    """
    Invalid input (4xx errors)
    """
    pass



import functools
import requests
from http import HTTPStatus
from json import JSONDecodeError
from typing import Optional, Dict, Any, Callable
from requests import Response
from gigapipe.exceptions import GigapipeClientError, GigapipeServerError
from gigapipe.api_client.api import Base


class GigapipeApi(object):
    """
    Gigapipe Python API
    """
    version: str = "v1"

    def __init__(self, url: str, *, client_id: str) -> None:
        """
        GigapipeApi Constructor
        :param url: URL of the api
        :param client_id: The Gigapipe Client ID
        """
        self.url: str = url
        self.access_token: Optional[str] = None
        self.refresh_token: Optional[str] = None
        self.client_id: Optional[str] = client_id

        # Authorizes the session upon creating an instance
        self.authorize()

    def authorize(self) -> None:
        """
        Authorizes the Gigapipe Python Client
        """
        from gigapipe import client_secret, client_prefix
        url: str = f"{self.url}/{self.__class__.version}/{client_prefix}/authorize"

        try:
            response: Response = requests.post(url, json={
                "client_id": self.client_id,
                "client_secret": client_secret
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        if response.status_code == HTTPStatus.NOT_FOUND:
            raise GigapipeClientError(
                status_code=HTTPStatus.NOT_FOUND,
                message="Resource Not found."
            )

        response: Dict[str, str] = response.json()
        self.access_token = response.get("access_token", None)
        self.refresh_token = response.get("refresh_token", None)

        if self.access_token is None:
            raise GigapipeClientError(
                status_code=HTTPStatus.UNAUTHORIZED,
                message="Invalid Authorization Token."
            )

    @staticmethod
    def should_refresh_access_token(response: Response) -> bool:
        """
        Whether the access token should be refreshed
        :param response: The response with a 401 Unauthorized error
        :return: true if the token needs refreshing, false otherwise
        """
        if response.status_code == HTTPStatus.UNAUTHORIZED:
            response_data: Dict[str, Any] = response.json()
            return response_data.get("description") == "Authorization token has expired"
        return False

    def refresh_access_token(self) -> None:
        """
        Refreshes the access token. This method is called automatically if the system detects that the
        current access token is expired
        """
        url: str = f"{self.url}/{self.__class__.version}/refresh"

        try:
            response = requests.post(url, json={
                "token": self.refresh_token
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        try:
            response_data: Dict[str, Any] = response.json()
        except JSONDecodeError as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Could not parse JSON Response: {e}"
            )
        self.access_token = response_data["access_token"]
        self.refresh_token = response_data["refresh_token"]

    @staticmethod
    def autorefresh_access_token(method: Callable) -> Callable:
        """
        Refreshes the access token upon receiving 401 Unauthorized when the token is expired
        :param method: The caller method which calls the decorator
        :return: A call to the wrapper function
        """

        @functools.wraps(method)
        def wrapper(obj: Base, *args, **kwargs) -> Any:
            """
            Checks the token and applies the logic that will return the response
            :param obj: The object that contains the caller function. It will always inherit from Base
            :return: The JSON parsed response
            """
            response: Response = method(obj, *args, **kwargs)

            if obj.api.should_refresh_access_token(response=response):
                obj.api.refresh_access_token()
                response = method(obj, *args, **kwargs)
            try:
                response_data: ... = response.json()
            except JSONDecodeError as e:
                raise GigapipeServerError(
                    status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                    message=f"Could not parse JSON Response: {e}"
                )
            return response_data

        return wrapper




import requests
from http import HTTPStatus
from typing import Dict
from requests import Response
from gigapipe.exceptions import GigapipeServerError
from gigapipe.api_client.api import Base
from gigapipe.api_client.gigapipe_api import GigapipeApi


class Users(Base):
    """
    Users Class
    """

    def __init__(self, api):
        """
        Users Constructor
        :param api: The API instance
        """
        super(Users, self).__init__(api)

    @GigapipeApi.autorefresh_access_token
    def get_info(self) -> Response:
        """
        Obtains the user-in-session info
        :return: A dictionary containing the user info
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/users/me"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def change_password(self, password_object: Dict[str, str]) -> Response:
        """
        Changes the user password
        :return: A message response
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/users/change-password"

        try:
            response: Response = requests.patch(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            }, json=password_object)
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def update_name(self, user_object: Dict[str, str]) -> Response:
        """
        Changes the user first and last name
        :return: A dictionary containing the user-in-session info with the new name
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/users"

        try:
            response: Response = requests.patch(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            }, json=user_object)
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def get_permissions(self) -> Response:
        """
        Obtains the user-in-session info
        :return: A dictionary containing the user permissions
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/users/me/permissions"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def get_upcoming_invoice(self) -> Response:
        """
        Obtains the user organization upcoming invoice
        :return: A dictionary containing the invoice values
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/users/upcoming-invoice"

        try:
            response: Response = requests.get(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response

    @GigapipeApi.autorefresh_access_token
    def delete_user(self) -> Response:
        """
        Deletes the gigapipe session user
        :return: The user payload after it's been deleted
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/users"

        try:
            response: Response = requests.delete(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            })
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response




import time

import gigapipe

gigapipe.client_secret = "bg_Brq_low-9W9PIb0ZQj1AFdlTblUO7HQcmwBWETrZsuRbXlH147ur8caRo7dNeATyx2252dM2KCz65J-K7OQ"

gigapipe = gigapipe.GigapipeClient(client_id="gigapipe-TCyAANYS9zS3cwa0lChfew")


### /users/me (get)

print('\n### User get info')
print(gigapipe.users.get_info())

# ### /users/change-password (patch)
#
# password_object = {
#     "new_password": "Qwer123?",
#     "new_password_verification": "Qwer123?",
#     "old_password": "Iwimd546?"
# }
#
# response = gigapipe.users.change_password(password_object)
#
# print('\n### Users change password')
# print(response)
#
#
# ### /users (patch)
#
# user_object = {
#     "first_name": "Barraca",
#     "last_name": "Hernandez",
# }
#
# response = gigapipe.users.update_name(user_object)
#
# print('\n### User update name')
# print(response)
#
# ### /users/me/permissions (get)
# print('\n### User permissions')
# print(gigapipe.users.get_permissions())
#
#
# ### /users/upcoming-invoice (get)
# print('\n### User invoice')
# print(gigapipe.users.get_upcoming_invoice())
#
#
# # ### /users (delete)
# # print('\n### Delete User')
# # print(gigapipe.users.delete_user())
#
#
# ### /organizations/users (get)
# print('\n### Organizations users')
# print(gigapipe.organizations.get_users())
#
# ### /organizations/invites (get)
# print('\n### Organizations invites')
# print(gigapipe.organizations.get_invites())
#
# # ### /organizations (delete)
# # print('\n### Organizations delete')
# # print(gigapipe.organizations.delete_organization())
#
# ### /users/upcoming-invoice (get)
# print('\n### Organizations invoice')
# print(gigapipe.organizations.get_upcoming_invoice())
#
# ### /invites (post)
#
# invite_object = {
#     "email": "asdsa@asdasd.es",
#     "organization_name": "gigapipe",
# }
#
# response = gigapipe.invites.send_invite(invite_object)
#
# print('\n### Invites send')
# print(response)
#
#
# ### /invites (delete)
# response = gigapipe.invites.delete_invite("asdsa@asdasd.es")
#
# print('\n### Invites delete')
# print(response)
#
#
# ### /roles/switch (post)
#
# role_object = {
#     "user_email": "test@a.com",
#     "role_name": "Admin"
# }
#
# response = gigapipe.roles.switch(role_object)
#
# print('\n### Roles switch')
# print(response)
#
#
# ### /machines (get)
#
# response = gigapipe.root.get_machines(provider_id=2, region_id=4)
#
# print('\n### Get machines')
# print(response)
#
# ### /machines/{id} (get)
#
# response = gigapipe.root.get_machine(machine_id=1)
#
# print('\n### Get Machine')
# print(response)
#
#
# ### /providers (get)
#
# response = gigapipe.root.get_providers()
#
# print('\n### Get Providers')
# print(response)
#
#
# ### /regions (get)
#
# response = gigapipe.root.get_regions(provider_id=2)
#
# print('\n### Get Regions')
# print(response)
#
# ### /disk-types (get)
#
# response = gigapipe.root.get_disk_types(provider_id=2, region_id=4)
#
# print('\n### Disk types')
# print(response)
#
#
# # ### /clusters (post)
# #
# # response = gigapipe.clusters.create_cluster({
# #     "name": "Cluster John",
# #     "machine_id": 1,
# #     "clickhouse": {
# #         "shards": 3,
# #         "replicas": 1,
# #         "disks": [
# #             {
# #                 "type": "gp2",
# #                 "size": 150.0,
# #                 "unit": "GB"
# #             }
# #         ],
# #         "admin": {
# #             "username": "john",
# #             "password": "johnpw"
# #         }
# #     },
# #     "provider_id": 2,
# #     "region_id": 4
# # })
# #
# # print('\n### Create cluster')
# # print(response)
#
# # ### /clusters (get)
# #
# # response = gigapipe.clusters.get_clusters()
# #
# # print('\n### Get clusters')
# # print(response)
# #
# # ### /clusters/{slug} (get)
# #
# # response = gigapipe.clusters.get_cluster(cluster_slug='cluster-john')
# #
# # print('\n### Get cluster')
# # print(response)
# #
# # ### /clusters/{slug}/query (get)
# #
# # response = gigapipe.clusters.get_cluster(cluster_slug='cluster-john')
# #
# # print('\n### Get cluster')
# # print(response)
#
# # ### /clusters/{slug}/query (get)
# #
# # response = gigapipe.clusters.query_cluster(cluster_slug='cluster-john', query='SELECT now()')
# #
# # print('\n### Query cluster')
# # print(response)
# #
# #
# # ### /clusters/{slug}/metadata (get)
# #
# # response = gigapipe.clusters.get_metadata(cluster_slug='cluster-john')
# #
# # print('\n### Query metadata')
# # print(response)
#
# # ### /clusters/{slug}/stop (get)
# #
# # response = gigapipe.clusters.stop_cluster(cluster_slug='cluster-john')
# #
# # print('\n### Stop cluster')
# # print(response)
# #
# # time.sleep(10)
# #
# # ### /clusters/{slug}/start (get)
# #
# # response = gigapipe.clusters.resume_cluster(cluster_slug='cluster-john')
# #
# # print('\n### Start')
# # print(response)
#
# # ### /clusters/{slug}/scale/nodes (get)
# #
# # response = gigapipe.clusters.scale_nodes('cluster-john', payload={
# #     "new_shards": 1,
# #     "new_replicas": 1
# # })
# #
# # print('\n### Scale')
# # print(response)
#
# ### /clusters/{slug}/disks (patch)
#
# response = gigapipe.clusters.add_disks('cluster-john', payload=[{
#     "autoscale_type": "%",
#     "autoscale_value": 10,
#     "type": "gp2",
#     "size": 10.0,
#     "unit": "GB"
# }])
#
# print('\n### Add disks')
# print(response)
#
# ### /clusters/{slug}/machines (patch)
#
# response = gigapipe.clusters.autoscale_disk('cluster-john', {
#     "autoscale_type": "GiB",
#     "autoscale_value": 15,
#     "id": 8
# })
#
# print('\n### Autoscale disk')
# print(response)
#
# ### Get autoscale
#
# response = gigapipe.clusters.get_autoscaling('cluster-john', disk_id=8)
#
# print('\n### Autoscale disk GET')
# print(response)
#
# ### Get autoscale
#
# response = gigapipe.clusters.delete_autoscaling('cluster-john', disk_id=8)
#
# print('\n### Delete autoscaling')
# print(response)
#
# ### /clusters/{slug}/machines (patch)
#
# response = gigapipe.clusters.change_machine('cluster-john', machine_id=2)
#
# print('\n### Change machine')
# print(response)
#
# ### /clusters/costs/all (get)
#
# response = gigapipe.clusters.get_costs()
#
# print('\n### Get costs')
# print(response)
#
# ### /clusters/{slug}/transferl (get)
#
# response = gigapipe.clusters.transfer_cluster(cluster_slug='cluster-john', email='target_user@gmail.com')
#
# print('\n### Transfer')
# print(response)
#
#
# ### delete cluster
#
# response = gigapipe.clusters.delete_cluster(cluster_slug='cluster-john')
#
# print('\n### delete cluster')
# print(response)


from typing import Dict
import requests
from http import HTTPStatus
from requests import Response
from gigapipe.exceptions import GigapipeServerError
from gigapipe.api_client.api import Base
from gigapipe.api_client.gigapipe_api import GigapipeApi


class Roles(Base):
    """
    Roles Class
    """

    def __init__(self, api):
        """
        Roles Constructor
        :param api: The API instance
        """
        super(Roles, self).__init__(api)

    @GigapipeApi.autorefresh_access_token
    def switch(self, role_object: Dict[str, str]) -> Response:
        """
        Switches the role of the passed user
        :return: A message response or an error dict
        """
        url: str = f"{self.api.url}/{self.api.__class__.version}/roles/switch"

        try:
            response: Response = requests.post(url, headers={
                "Authorization": f"Bearer {self.api.access_token}"
            }, json=role_object)
        except requests.RequestException as e:
            raise GigapipeServerError(
                status_code=HTTPStatus.INTERNAL_SERVER_ERROR,
                message=f"Internal Server Error: {e}"
            )
        return response



from setuptools import setup, find_packages

VERSION = "0.1.11"

classifiers = [
    'Development Status :: 5 - Production/Stable',
    'Intended Audience :: Developers',
    'Operating System :: Unix',
    'Operating System :: MacOS :: MacOS X',
    'Operating System :: Microsoft :: Windows :: Windows 10',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 3'
]

setup(
    name='gigapipe',
    version=VERSION,
    description='Gigapipe Python Client',
    long_description_content_type='text/markdown',
    long_description=open('README.md').read() + '\n\n' + open('CHANGELOG.txt').read(),
    url='https://www.gigapipe.com',
    project_urls={
        "Github": "https://github.com/gigapipehq/gigapipe-python-client"
    },
    author='Gigapipe',
    author_email='info@gigapipe.com',
    license='MIT',
    classifiers=classifiers,
    keywords='bigdata, gigapipe, clickhouse',
    packages=find_packages(),
    install_requires=["requests==2.27.1"]
)

#  python3 setup.py sdist bdist_wheel
#  twine upload --repository-url https://upload.pypi.org/legacy/ dist/*



from gigapipe.api_client.api import (
    Users,
    Organizations,
    Invites,
    Roles,
    Root,
    Clusters,
    Clickhouse,
    Imports,
    Integrations
)
from gigapipe.api_client.gigapipe_api import GigapipeApi


class GigapipeClient(object):
    """
    Gigapipe Python Client
    """

    gigapipe_url: str = "https://api.gigapipe.com"

    def __init__(self, client_id: str):
        """
        GigapipeClient Constructor
        :param client_id: The Gigapipe Client ID
        """
        self.api = GigapipeApi(self.__class__.gigapipe_url, client_id=client_id)

        # Client Objects
        self.__root = Root(self.api)
        self.__users = Users(self.api)
        self.__organizations = Organizations(self.api)
        self.__invites = Invites(self.api)
        self.__roles = Roles(self.api)
        self.__clusters = Clusters(self.api)
        self.__clickhouse = Clickhouse(self.api)
        self.__imports = Imports(self.api)
        self.__integrations = Integrations(self.api)

    @property
    def root(self):
        """
        Root instance
        :return: The instance that handles all the root-related calls
        """
        return self.__root

    @property
    def users(self):
        """
        Users instance
        :return: The instance that handles all the user-related calls
        """
        return self.__users

    @property
    def organizations(self):
        """
        Organizations instance
        :return: The instance that handles all the organization-related calls
        """
        return self.__organizations

    @property
    def invites(self):
        """
        Invites instance
        :return: The instance that handles all the invites-related calls
        """
        return self.__invites

    @property
    def roles(self):
        """
        Roles instance
        :return: The instance that handles all the roles-related calls
        """
        return self.__roles

    @property
    def clusters(self):
        """
        Clusters instance
        :return: The instance that handles all the clusters-related calls
        """
        return self.__clusters

    @property
    def clichhouse(self):
        """
        Clickhouse instance
        :return: The instance that handles all the clickhouse-related calls
        """
        return self.__clickhouse

    @property
    def imports(self):
        """
        Imports instance
        :return: The instance that handles all the clickhouse-related calls
        """
        return self.__imports

    @property
    def integrations(self):
        """
        Integrations instance
        :return: The instance that handles all the clickhouse-related calls
        """
        return self.__integrations

